unit OpenGlPanel;

interface

uses
  SysUtils, Classes, Controls, ExtCtrls, OpenGl, Windows,
  Messages, Variants, Graphics, Forms,
  Dialogs;


type
  TOpenGlBasePanel = Class ( TPanel )
  private
    Initialized  : Boolean;
    FGlDraw      : TNotifyEvent;
    rc           : HGLRC;
    dc           : HDC;

    procedure Idle(Sender: TObject; var Done: Boolean);
  public
    hApplication : TApplication;
    Procedure glInit; virtual;
    procedure Paint; override;
    destructor Destroy; override;

    constructor Create(AOwner: TComponent); override;
  protected
    procedure Resize; override;
  published
    Property glDraw : TNotifyEvent read FGlDraw write FGlDraw;
    Property Onkeydown;
  end;

  TOpenGlPanel = class(TOpenGlBasePanel)
  private
    fWireFrame : Boolean;
    procedure SetWireFrame(const Value: Boolean);
    { Private declarations }
  protected
  public
    procedure Paint; override;
    constructor Create(AOwner: TComponent); override;
  published
    Property WireFrame : Boolean Read fWireFrame Write SetWireFrame;
  end;

procedure Register;

implementation

//------------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('Shadow_tj', [TOpenGlPanel]);
end;

//------------------------------------------------------------------------------

{ TOpenGlPanel }

//------------------------------------------------------------------------------

procedure TOpenGlBasePanel.glInit();
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  Initialized := True;

  dc:=GetDC( Self.Handle );
  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
  pfd.iPixelType:=PFD_TYPE_RGBA;
  pfd.cColorBits:=32;
  pf :=ChoosePixelFormat(dc, @pfd);
  SetPixelFormat(dc, pf, @pfd);
  rc :=wglCreateContext(dc);
  wglMakeCurrent(dc,rc);

  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_SMOOTH);
  glClearDepth(1.0);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  // Tunnel
  glClearColor(0.0, 0.0, 0.0, 0.0); 	                 // Black Background
  glShadeModel(GL_SMOOTH);                             // Enables Smooth Color Shading
  glClearDepth(1.0);                                   // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                             // Enable Depth Buffer
  glDepthFunc(GL_LESS);		                             // The Type Of Depth Test To Do
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
  glEnable(GL_TEXTURE_2D);

  Resize;
  Application.OnIdle := Idle;
end;

//------------------------------------------------------------------------------

constructor TOpenGlBasePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  //hApplication := Application;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Initialized := False;
end;

//------------------------------------------------------------------------------

destructor TOpenGlBasePanel.Destroy;
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);

  inherited Destroy;
end;

//------------------------------------------------------------------------------

procedure TOpenGlBasePanel.Idle(Sender: TObject; var Done: Boolean);
begin
  glDraw(Self);
  SwapBuffers(DC);
  Done := False;
end;

//------------------------------------------------------------------------------

procedure TOpenGlBasePanel.Resize;
begin
  inherited;
  glViewport(0, 0, Self.Width, Self.Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, Self.Width/Self.Height, 1.0, 500.0);
  glMatrixMode(GL_MODELVIEW);
end;

//------------------------------------------------------------------------------

procedure TOpenGlBasePanel.Paint;
begin
  inherited;
  if csDesigning in ComponentState then
	with inherited Canvas do
	begin
	  Pen.Style := psDash;
	  Brush.Style := bsClear;
	  Rectangle(0, 0, Width, Height);
	end;
end;

//------------------------------------------------------------------------------

procedure TOpenGlPanel.Paint;
begin
  inherited;
  // Wire Frame Mode
  if Wireframe = TRUE then
    glPolygonmode(GL_FRONT_AND_BACK, GL_LINE)
  else
    glPolygonmode(GL_FRONT, GL_FILL);
end;

//------------------------------------------------------------------------------

constructor TOpenGlPanel.Create(AOwner: TComponent);
begin
  inherited;
  Cursor := crNone;
end;

//------------------------------------------------------------------------------

procedure TOpenGlPanel.SetWireFrame(const Value: Boolean);
begin
  fWireFrame := Value;
  Paint;
end;

//------------------------------------------------------------------------------

end.







